from .utilities import unpackTimecode, findEvenSplit


class LyricLine:
    """An object that holds a lyric line and it's time"""

    def __init__(self, timecode, text=""):
        self.hours = 0
        self.minutes, self.seconds, self.milliseconds = unpackTimecode(timecode)
        self.time = sum([(self.hours * 3600), (self.minutes * 60),
                         self.seconds, (self.milliseconds / 1000)])
        self.text = text

    def shift(self, minutes=0, seconds=0, milliseconds=0):
        """Shift the timecode by the given amounts"""

        self.addMillis(milliseconds)
        self.addSeconds(seconds)
        self.addMinutes(minutes)

    def addMillis(self, milliseconds):
        summation = self.milliseconds + milliseconds
        if summation > 999 or summation < 0:
            self.milliseconds = (self.milliseconds + milliseconds) % 1000
            self.addSeconds(int((self.milliseconds + milliseconds) / 1000))
        else:
            self.milliseconds = summation

    def addSeconds(self, seconds):
        summation = self.seconds + seconds
        if summation > 59 or summation < 0:
            self.seconds = (self.seconds + seconds) % 60
            self.addMinutes(int((self.seconds + seconds) / 60))
        else:
            self.seconds = summation

    def addMinutes(self, minutes):
        summation = self.minutes + minutes
        if summation > 59 or summation < 0:
            self.minutes = (self.minutes + minutes) % 60
            self.addHours(int((self.minutes + minutes) / 60))
        else:
            self.minutes = self.minutes + minutes

    def addHours(self, hours):
        summation = self._hours + hours
        if summation > 23:
            self.hours = 23
        elif summation < 0:
            self.hours = 0
            self.minutes = 0
            self.seconds = 0
            self.milliseconds = 0
        else:
            self._hours = summation

    def __lt__(self, other):
        """For sorting instances of this class"""
        return self.time < other.time


class Lyrics(list):
    """A list that holds the contents of the lrc file"""

    def __init__(self, items=None):

        super().__init__()
        if items is None:
            items = []
        self.artist = ""
        self.album = ""
        self.title = ""
        self.author = ""
        self.length = ""
        self.offset = 0

        self.extend(items)

    def toSRT(self):
        """Returns an SRT string of the LRC data"""

        if not self[-1].text.rstrip() == "":
            timecode = ''.join(['[', str(self[-1].minutes), ':',
                                str(self[-1].seconds), '.',
                                str(self[-1].milliseconds), ']'])
            end_line = LyricLine(timecode, "")
            end_line.shift(seconds=5)
            self.append(end_line)

        output = []
        for i in range(len(self) - 1):
            if not self[i].text == '':
                srt = str(i) + '\n'
                start_hours = "%02d" % self[i].hours
                start_min = "%02d" % self[i].minutes
                start_sec = "%02d" % self[i].seconds
                start_milli = "%03d" % self[i].milliseconds
                start_timecode = ''.join([start_hours, ':', start_min,
                                          ':', start_sec, ',', start_milli])
                end_hours = "%02d" % self[i + 1].hours
                end_min = "%02d" % self[i + 1].minutes
                end_sec = "%02d" % self[i + 1].seconds
                milliseconds = self[i + 1].milliseconds - 1
                end_milli = "%03d" % (0 if milliseconds < 0 else milliseconds)
                end_timecode = ''.join([end_hours, ':', end_min,
                                        ':', end_sec, ',', end_milli])

                srt = srt + start_timecode + ' --> ' + end_timecode + '\n'
                if len(self[i].text) > 31:
                    srt = srt + findEvenSplit(self[i].text) + '\n'
                else:
                    srt = srt + self[i].text + '\n'
                output.append(srt)

        return '\n'.join(output).rstrip()

    def toLRC(self):
        output = []
        if self.artist != "":
            output.append('[ar:' + self.artist + ']')
        if self.album != "":
            output.append('[al:' + self.album + ']')
        if self.title != "":
            output.append('[ti:' + self.title + ']')
        if self.author != "":
            output.append('[au:' + self.author + ']')
        if self.length != "":
            output.append('[length:' + self.length + ']')
        if self.offset != 0:
            output.append('[offset:' + str(self.offset) + ']')

        if output:
            output.append('')

        for i in self:
            minutes = "%02d" % i.minutes
            seconds = "%02d" % i.seconds
            milliseconds = ("%02d" % i.milliseconds)[0:2]

            lrc = ''.join(['[', minutes, ':', seconds, '.', milliseconds, ']'])
            lrc += i.text
            output.append(lrc)
        return '\n'.join(output).rstrip()



from datetime import datetime


def validateTimecode(timecode):
    """Checks if a string is a timecode of format [%M:%S.%f]"""

    try:
        unpackTimecode(timecode)
        return True

    except ValueError:
        return False


def unpackTimecode(timecode):
    """unpacks a timecode to minutes, seconds, and milliseconds"""

    if "." in timecode:
        x = datetime.strptime(timecode, '[%M:%S.%f]')
    else:
        x = datetime.strptime(timecode, '[%M:%S]')
    minutes = x.minute
    seconds = x.second
    milliseconds = int(x.microsecond / 1000)
    return minutes, seconds, milliseconds


def findEvenSplit(line):
    """
    Given a string, splits it into two evenly spaced lines
    """
    word_list = line.split(' ')
    differences = []
    for i in range(len(word_list)):
        group1 = ' '.join(word_list[0:i + 1])
        group2 = ' '.join(word_list[i + 1::])
        differences.append(abs(len(group1) - len(group2)))
    index = differences.index(min(differences))
    for i in range(len(word_list)):
        if i == index:
            group1 = ' '.join(word_list[0:i + 1])
            group2 = ' '.join(word_list[i + 1::])

    return ''.join([group1, '\n', group2]).rstrip()



import re

from .classes import Lyrics, LyricLine
from .utilities import validateTimecode

synced_line_regex = re.compile(r'^(\[[0-5]\d:[0-5]\d(\.\d\d)?\])+.*', flags=re.MULTILINE)


def parse(lrc):
    lines = lrc.split('\n')
    lyrics = Lyrics()
    items = []

    for line in lines:
        if line.startswith('[ar:'):
            lyrics.artist = line.rstrip()[4:-1].lstrip()

        elif line.startswith('[ti:'):
            lyrics.title = line.rstrip()[4:-1].lstrip()

        elif line.startswith('[al:'):
            lyrics.album = line.rstrip()[4:-1].lstrip()

        elif line.startswith('[by:'):
            lyrics.author = line.rstrip()[4:-1].lstrip()

        elif line.startswith('[length:'):
            lyrics.length = line.rstrip()[8:-1].lstrip()

        elif line.startswith('[offset:'):
            try:
                lyrics.offset = int(line.rstrip()[8:-1].lstrip())
            except ValueError:
                pass

        elif line.startswith('[re:'):
            lyrics.editor = line.rstrip()[4:-1].lstrip()

        elif line.startswith('[ve:'):
            lyrics.version = line.rstrip()[4:-1].lstrip()

        elif synced_line_regex.match(line):
            text = ""
            first = True
            for split in reversed(line.split(']')):
                if validateTimecode(split + "]"):
                    lyric_line = LyricLine(split + "]", text=text)
                    items.append(lyric_line)
                else:
                    if not first:
                        split += "]"
                    else:
                        first = False
                    text = split + text

    lyrics.extend(sorted(items))

    if not lyrics.offset == 0:
        millis = lyrics.offset

        minutes = int(millis / 60000)
        millis %= millis / 60000

        secs = int(millis / 1000)
        millis %= millis / 1000

        for line in lyrics:
            line.shift(minutes=minutes, seconds=secs, milliseconds=millis)

    return lyrics



from setuptools import setup

setup(
    name='pylrc',
    packages=['pylrc'],
    version='0.0.6',
    description='A library for parsing .lrc files',
    long_description=open('README.rst').read(),
    author='doakey3',
    author_email='pylrc.dmodo@spamgourmet.com',
    url='https://github.com/doakey3/pylrc',
    download_url='https://github.com/doakey3/pylrc/releases/download/0.0.6/pylrc.tar.gz',
    license='MIT',
)
